package de.zappbrannigan.objects;

import java.util.Vector;

/**
 * Holds objects of Asteroid for faster generation of new asteroids
 */
public class AsteroidPool {

	private Vector<Asteroid> asteroidPool = new Vector<Asteroid>();
	private final int MAX_SIZE = 100000;
	private final int MIN_SIZE = 100;
	
	/**	A sequence to create unique ids for each object. */
	private int objectIdSequence = 0;

	private static AsteroidPool me = null;
	
	/**
	 * Constructor
	 */
	private AsteroidPool() {
		
		// create 100000 asteroids first
		createNewAsteroids(MAX_SIZE);
	}
	
	
	/**
	 * Get an instance of the asteroid pool
	 * 
	 * @return Instance of Asteroid pool
	 */
	public static AsteroidPool getAsteroidPool() {
		
		if (me == null) {
			
			me = new AsteroidPool();
		}
		
		return me;
	}
	
	/**
	 * Returns a freshly initialized asteroid
	 * 
	 * @param x Position x
	 * @param y Position y
	 * @param size Size
	 * @param type Type
	 * @return Asteroid
	 */
	public Asteroid requestAsteroid(int x, int y, int size, int type) {
		
		if (asteroidPool.size() < MIN_SIZE) {
			
			createNewAsteroids(100);
		}
		
		Asteroid newAsteroid = asteroidPool.remove(asteroidPool.size() - 1);

		newAsteroid.initializeVariables(x, y, size, type);
		
		return newAsteroid;
	}

	/**
	 * Release the asteroid object
	 * 
	 * @param oldAsteroid
	 */
	public void releaseAsteroid(Asteroid oldAsteroid) {
		
		if (asteroidPool.size() < MAX_SIZE) {
			
			asteroidPool.add(oldAsteroid);
		}
	}
	
	private void createNewAsteroids(int count) {
		
		for (int i = 0; i < count; i++) {
			
			asteroidPool.add(new Asteroid(0, 0, 0, 0));
		}
	}
	
	/**
	 * Get a unique id for a new object.
	 * 
	 * @return A unique id.
	 */
	public int getNextObjectUid() {
		
		return ++objectIdSequence;
	}
}
