// ============================================================================
// File:               $File$
//
// Project:            
//
// Purpose:            
//
// Author:             Rammi
//
// Copyright Notice:   (c) 2008  Rammi (rammi@caff.de)
//                     This code is in the public domain.
//                     Use at own risk.
//                     No guarantees given.
//
// Latest change:      $Date$
//
// History:	       $Log$
//=============================================================================
package de.zappbrannigan.objects;

/**
 *  Some useful constants of the Asteroids game.
 *
 *  This class is part of a solution for a
 *  <a href="http://www.heise.de/ct/creativ/08/02/details/">competition by the German computer magazine c't</a>.
 */
public interface GameData
{
  /** Size of the vector ram. */
  final int VECTORRAM_SIZE = 1024;
  /** Extent of x and y coordinates. */
  final int EXTENT = 8192;
  /** Border above and below y because of 4/3 screen aspect ratio. */
  final int BORDER_Y = EXTENT/8;
  /** The screen extend in x direction. */
  final int EXTENT_X = EXTENT;
  /** The screen extent in y direction. */
  final int EXTENT_Y = EXTENT - 2*BORDER_Y;
  /** Half extent of x and y coordinates */
  final int HALF_EXTENT = EXTENT/2;
  /** Half extent in y direction */
  final int HALF_EXTENT_Y = EXTENT_Y/2;
  /** The minimal x coordinate used. */
  final int MIN_X = 0;
  /** The maximal x coordinate used. */
  final int MAX_X = EXTENT_X - 1;
  /** The minimal y coordinate used. */
  final int MIN_Y = BORDER_Y;
  /** The maximal y coordinate used. */
  final int MAX_Y = BORDER_Y + EXTENT_Y - 1;
  /** Hyperspace button pressed. */
  final int BUTTON_HYPERSPACE = 0x01;
  /** Fire button pressed. */
  final int BUTTON_FIRE       = 0x01 << 1;
  /** Thrust button pressed. */
  final int BUTTON_THRUST     = 0x01 << 2;
  /** Left button pressed. */
  final int BUTTON_RIGHT      = 0x01 << 3;
  /** Right button pressed. */
  final int BUTTON_LEFT       = 0x01 << 4;
  /** Start button pressed (requires mameaster patch). */
  final int BUTTON_START      = 0x01 << 5;
  /** Port used by mame. */
  final int MAME_PORT = 1979;
  /** Lifetime of torpedos in frames. */
  final int TORPEDO_LIFETIME = 68;
  /** Speed of torpedos in pixels per frame. */
  final int TORPEDO_SPEED = 64;
  /** Maximum range of a torpedo. */
  final int TORPEDO_RANGE = TORPEDO_SPEED * TORPEDO_LIFETIME;
  /** Angle of one turn */
  final double ONE_TURN_ANGLE = 2 * Math.PI / 256 * 3;
  /** List of possible turning directions. */
  public static enum Direction {
	  /** Turn to left. */
	  LEFT,
	  /** Turn to right. */
	  RIGHT,
	  /** Don't turn. */
	  NEUTRAL
  }
}
