/**
 * 
 */
package de.zappbrannigan.objects;

import java.awt.Point;

import de.zappbrannigan.objects.GameData.Direction;

/**
 * @author gromit
 *
 */
public class ShootingSolution {

	/** The point where the bullet should hit it's target. */
	private Point hitPoint;
	/** The rotation byte / direction of this shot. */
	private int rotByte;
	/** The time needed for reaching the correct rotation. */
	private int turningTime;
	/** The time to wait before shooting. */
	private int waitingTime;
	/** The time the torpedo needs to reach hitpoint. */
	private int shootingTime;
	/** The id of the target. */
	private int targetUid;
	/** The direction to turn into. */
	private Direction turningDirection = Direction.NEUTRAL;
	/** Number of shots fired according to this shooting solution. */
	private int shotsFired = 0;
	/** Number of shots planned to fire. */ 
	private int shotsPlanned = 1;
	
	/**
	 * Constructor.
	 * 
	 * @param targetUid
	 * @param hitPoint
	 * @param rotByte
	 * @param turningTime
	 * @param waitingTime
	 * @param shootingTime
	 */
	public ShootingSolution(int targetUid, Point hitPoint, int rotByte, 
			int turningTime, int waitingTime, int shootingTime) {
		
		this.targetUid = targetUid;
		this.hitPoint = hitPoint;
		this.rotByte = rotByte;
		this.turningTime = turningTime;
		this.waitingTime = waitingTime;
		this.shootingTime = shootingTime;
	}
	
	/**
	 * @return the hitPoint
	 */
	public Point getHitPoint() {
		
		return hitPoint;
	}
	
	/**
	 * @param hitPoint the hitPoint to set
	 */
	public void setHitPoint(Point hitPoint) {
		
		this.hitPoint = hitPoint;
	}
	
	/**
	 * @return the rotByte
	 */
	public int getRotByte() {
		
		return rotByte;
	}
	
	/**
	 * @param rotByte the rotByte to set
	 */
	public void setRotByte(int rotByte) {
		
		this.rotByte = rotByte;
	}

	/**
	 * @return the turningTime
	 */
	public int getTurningTime() {
		
		return turningTime;
	}
	
	/**
	 * @param turningTime the turningTime to set
	 */
	public void setTurningTime(int turningTime) {
		
		this.turningTime = turningTime;
	}
	
	/**
	 * Adds a given delta to the turning time.
	 * 
	 * @param delta
	 */
	public void addTurningTime(int delta) {
		
		this.turningTime += delta;
		
		// handle the case that we turned too far
		if (turningTime < 0) {
			// by reversing this plans direction
			reverseTurningDirection();
			// and correcting the number of steps
			turningTime = Math.abs(turningTime);
		}
	}
	
	/**
	 * @return the waitingTime
	 */
	public int getWaitingTime() {
		
		return waitingTime;
	}
	
	/**
	 * @param waitingTime the waitingTime to set
	 */
	public void setWaitingTime(int waitingTime) {
		
		this.waitingTime = waitingTime;
	}
	
	/**
	 * Adds a given delta to the waiting time.
	 * 
	 * @param delta
	 */
	public void addWaitingTime(int delta) {
		
		this.waitingTime += delta;
	}
	
	/**
	 * @return the shootingTime
	 */
	public int getShootingTime() {
		
		return shootingTime;
	}
	
	/**
	 * @param shootingTime the shootingTime to set
	 */
	public void setShootingTime(int shootingTime) {
		
		this.shootingTime = shootingTime;
	}
	
	/**
	 * Returns the full time needed for placing this shot.
	 * 
	 * @return
	 */
	public int getTotalTime() {
		
		return waitingTime + turningTime + shootingTime;
	}

	/**
	 * @return the targetUid
	 */
	public int getTargetUid() {
		
		return targetUid;
	}

	/**
	 * @param targetUid the targetUid to set
	 */
	public void setTargetUid(int targetUid) {
		
		this.targetUid = targetUid;
	}
	
	public String toString() {
		
		return String.format("ShootingSolution: targetUid=%d, rotation=%d, t=%d, w=%d, s=%d", 
				targetUid, rotByte, turningTime, waitingTime, shootingTime); 
	}

	/**
	 * @return the turningDirection
	 */
	public Direction getTurningDirection() {
		return turningDirection;
	}

	/**
	 * @param turningDirection the turningDirection to set
	 */
	public void setTurningDirection(Direction turningDirection) {
		this.turningDirection = turningDirection;
	}
	
	private void reverseTurningDirection() {
		
		if (turningDirection == Direction.LEFT) {
			turningDirection = Direction.RIGHT;
		}
		else if (turningDirection == Direction.RIGHT) {
			turningDirection = Direction.LEFT;
		}
	}

	/**
	 * @return the shotsFired
	 */
	public int getShotsFired() {
		
		return shotsFired;
	}

	/**
	 * @param shotsFired the shotsFired to set
	 */
	public void setShotsFired(int shotsFired) {
		
		this.shotsFired = shotsFired;
	}
	
	/**
	 * Increases the counter of shots fired.
	 */
	public void incShotsFired() {
		
		this.shotsFired++;
	}

	/**
	 * @return the shotsPlanned
	 */
	public int getShotsPlanned() {
		return shotsPlanned;
	}

	/**
	 * @param shotsPlanned the shotsPlanned to set
	 */
	public void setShotsPlanned(int shotsPlanned) {
		this.shotsPlanned = shotsPlanned;
	}
}
