package de.zappbrannigan.util;
import de.zappbrannigan.objects.Asteroid;
import de.zappbrannigan.objects.AsteroidPool;

/**
 * Test class for checking whether the asteroid pool is faster 
 * than object generation
 */
public class TestCreationVersusPool {

	private static AsteroidPool asteroidPool = AsteroidPool.getAsteroidPool();
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		// warm machen
		for (int i = 0; i < 10000000; i++) {
			
			Asteroid ast = asteroidPool.requestAsteroid(0, 0, 0, 0);
			useAsteroid(ast);
			asteroidPool.releaseAsteroid(ast);
		}
		
		for (int i = 0; i < 10; i++) {
			long startTime = System.currentTimeMillis();
			System.out.println(startTime);
			
			testCreation(100000);
			
			long duration = System.currentTimeMillis() - startTime;
			System.out.println("Creation: " + duration + " millisecs");
	
			startTime = System.currentTimeMillis();
			testPool(100000);
	
			duration = System.currentTimeMillis() - startTime;
			System.out.println("Pool: " + duration + " millisecs");
		}
	}
	
	private static void testCreation(int count) {
		
		for (int i = 0; i < count; i++) {
			
			Asteroid ast = new Asteroid(0, 0, 0, 0);
			useAsteroid(ast);
		}
	}

	private static void testPool(int count) {
		
		for (int i = 0; i < count; i++) {
			
			Asteroid ast = asteroidPool.requestAsteroid(0, 0, 0, 0);
			useAsteroid(ast);
			asteroidPool.releaseAsteroid(ast);
		}
	}
	
	private static void useAsteroid(Asteroid ast) {
		
		ast.setLocation(100, 100);
		ast.setVelocity(2, 2);
		ast.setAge(100);
	}
}
